/*
    This file is a part of the estlib C++ library.
    Copyright (C) 1998  Eric S. Tiedemann

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// encapsulates pthreads mutexe (de)acquisition using constructors 
// and destructors

#ifndef _ESTLIB_MUTEX_H
#define _ESTLIB_MUTEX_H

extern "C"
{
#ifdef MUTEX_DEBUG
#include <stdio.h>
#endif
#include <pthread.h>
}

namespace estlib
{
  class mutex
  {
    pthread_mutex_t *m;
    
  public:
    
    void
    wait(pthread_cond_t *c)
      {
	pthread_cond_wait(c, m);
      }
    
    mutex(pthread_mutex_t *m)
      : m(m)
      {
#ifdef MUTEX_DEBUG
	fprintf(stderr, "locking mutex %p\n", m);
#endif
	pthread_mutex_lock(m);
#ifdef MUTEX_DEBUG
	fprintf(stderr, "mutex %p locked\n", m);
#endif
      }
    
    ~mutex()
      {
#ifdef MUTEX_DEBUG
	fprintf(stderr, "unlocking mutex %p\n", m);
#endif
	pthread_mutex_unlock(m);
#ifdef MUTEX_DEBUG
	fprintf(stderr, "mutex %p unlocked\n", m);
#endif
      }
  };

  class nomutex
  {
    pthread_mutex_t *m;
    
  public:
    
    void
    wait(pthread_cond_t *c)
      {
	pthread_cond_wait(c, m);
      }
    
    nomutex(pthread_mutex_t *m)
      : m(m)
      {
#ifdef MUTEX_DEBUG
	fprintf(stderr, "unlocking mutex %p\n", m);
#endif
	pthread_mutex_unlock(m);
#ifdef MUTEX_DEBUG
	fprintf(stderr, "mutex %p unlocked\n", m);
#endif
      }
    
    ~nomutex()
      {
#ifdef MUTEX_DEBUG
	fprintf(stderr, "locking mutex %p\n", m);
#endif
	pthread_mutex_lock(m);
#ifdef MUTEX_DEBUG
	fprintf(stderr, "mutex %p locked\n", m);
#endif
      }
  };
}

#endif /* _ESTLIB_MUTEX_H */
